import yaml from 'js-yaml'
import markdown, { profiles as markdownProfiles } from '../transformers/markdown'
import rehype, { profiles as rehypeProfiles } from '../transformers/rehype'
import retext, { profiles as retextProfiles } from '../transformers/retext'
import { testRule } from 'skypager-util'
import visit from 'unist-util-visit'
import { lazy } from 'skypager-util'
import omit from 'lodash/omit'

export const rules = [
  /\.md$/
]

export const testDocument = (doc) => (
  typeof rules.find(rule => testRule(rule, doc.fileBaseName)) !== 'undefined'
)

export const decorate = (doc) => {
  doc.lazy('exportables', () => doc.createExportable())
  

  Object.assign(doc, {
    getTransforms() {
      return {
        markdown,
        rehype,
        retext,
        markdownProfiles: Object.keys(markdownProfiles),
        rehypeProfiles: Object.keys(rehypeProfiles),
        retextProfiles: Object.keys(retextProfiles),
      }
    },

    getUtils() {
      return {
        filter: require('unist-util-filter'),
        findAfter: require('unist-util-find-after'),
        findAllAfter: require('unist-util-find-all-after'),
        findAllBefore: require('unist-util-find-all-before'),
        index: require('unist-util-index'),
        map: require('unist-util-map'),
        parents: require('unist-util-parents'),
        position: require('unist-util-position'),
        removePosition: require('unist-util-remove-position'),
        select: require('unist-util-select'),
        source: require('unist-util-source'),
        visit: require('unist-util-visit'),
        reporter: require('vfile-reporter'),
        rehypeDocument: require('rehype-document'),
        toString: require('mdast-util-to-string'),
      }
    },

    createExportable(options = {}) {
      const ast = doc.markdown(options.profile || 'parser', options)
      const html = doc.markdown('html', options)

      const exportable = {
        id: doc.id,
        visit (fn) {
          return visit(exportable.ast, fn)
        },
        get file() {
          return omit(doc.file, 'contents')
        }
      }

      lazy(exportable, 'contents', () => doc.readFile().getFileContents())
      lazy(exportable, 'ast', () => ast.parse(exportable.contents))
      lazy(exportable, 'html', () => html.stringify(exportable.ast))
      lazy(exportable, 'frontmatter', () => {
        const { children = [] } = exportable.ast
        const head = children[0] || {}

        return head && head.type === 'yaml' && head.value
          ? yaml.load(head.value)
          : {}
      })

      return exportable
    },

    readFrontmatter(options = {}) {
      const { contents = doc.getFileContents() } = options
      const parser = doc.markdown(options.parser || 'parser')
      const { children = [] } = parser.parse(contents)
      
      if (children[0] && children[0].type === 'yaml') {
        return yaml.load(children[0].value || '')
      }

      return {}
    },

    retextAnalysis(options = {}) {
      const profile = options.profile || 'rehype'
      const parser = doc.markdown(options.parser || 'parser', options)
      const contents = options.contents || doc.getFileContents()
      const ast = options.ast || parser.parse(contents)

      return doc.retext('retext', options).process(ast)
    }, 

    rehypeDocument(options = {}) {
      const parser = doc.markdown(options.parser || 'parser', options)
      const contents = options.contents || doc.getFileContents()
      const ast = options.ast || parser.parse(contents)

      return doc.rehype(profile, options).run(ast)     
    },

    remarkAST(options = {}) {
      const profile = options.profile || 'parser'
      const contents = options.contents || this.getFileContents()

      return doc.markdown(profile, options).parse(contents)    
    },

    remarkHTML(options = {}) {
      const profile = options.profile || 'github'
      const parser = options.parser || doc.markdown(profile, options)
      const contents = options.contents || this.getFileContents()
      const ast = options.ast || parser.parse(contents)
      const html = 'html'

      return doc.markdown(html, options).stringify(ast) 
    },

    retext(profile = 'retext', options) {
      return retext({
        profile,
        ...options,
      })
    },

    rehype(profile = 'rehype', options) {
      return rehype({
        profile,
        ...options,
      })
    },

    markdown(profile = 'parser', options) {
      return markdown({
        profile,
        ...options
      })
    }
  })

  return doc
}

export default decorate
